@@ -118,14 +118,14 @@ struct _garray
     char x_hidename;        /* don't print name above graph */
 };
 
-static t_pd *garray_arraytemplatecanvas;
-static char garray_arraytemplatefile[] = "\
+static t_pd *garray_arraytemplatecanvas;  /* written at setup w/ global lock */
+static const char garray_arraytemplatefile[] = "\
 canvas 0 0 458 153 10;\n\
 #X obj 43 31 struct float-array array z float float style\n\
 float linewidth float color;\n\
 #X obj 43 70 plot z color linewidth 0 0 1 style;\n\
 ";
-static char garray_floattemplatefile[] = "\
+static const char garray_floattemplatefile[] = "\
 canvas 0 0 458 153 10;\n\
 #X obj 39 26 struct float float y;\n\
 ";
@@ -768,7 +768,7 @@ static void garray_save(t_gobj *z, t_binbuf *b)
     garray_savecontentsto(x, b);
 }
 
-t_widgetbehavior garray_widgetbehavior =
+const t_widgetbehavior garray_widgetbehavior =
 {
     garray_getrect,
     garray_displace,
