@@ -16,19 +16,10 @@
 #define TIMEUNITPERMSEC (32. * 441.)
 #define TIMEUNITPERSECOND (TIMEUNITPERMSEC * 1000.)
 
-#ifndef THREAD_LOCKING
-#define THREAD_LOCKING 1
-#endif
-
-#if THREAD_LOCKING
-#include "pthread.h"
-#endif
-
 #define SYS_QUIT_QUIT 1
 #define SYS_QUIT_RESTART 2
 static int sys_quit;
 
-int sys_schedblocksize = DEFDACBLKSIZE;
 int sys_usecsincelastsleep(void);
 int sys_sleepgrain;
 
@@ -104,7 +95,8 @@ void clock_delay(t_clock *x, double delaytime)
 {
     clock_set(x, (x->c_unit > 0 ?
         pd_this->pd_systime + x->c_unit * delaytime :
-            pd_this->pd_systime - (x->c_unit*(TIMEUNITPERSECOND/sys_dacsr)) * delaytime));
+            pd_this->pd_systime -
+                (x->c_unit*(TIMEUNITPERSECOND/STUFF->st_dacsr)) * delaytime));
 }
 
     /* set the time unit in msec or (if 'samps' is set) in samples.  This
@@ -123,7 +115,7 @@ void clock_setunit(t_clock *x, double timeunit, int sampflag)
         /* figure out time left in the units we were in */
     timeleft = (x->c_settime < 0 ? -1 :
         (x->c_settime - pd_this->pd_systime)/((x->c_unit > 0)? x->c_unit :
-            (x->c_unit*(TIMEUNITPERSECOND/sys_dacsr))));
+            (x->c_unit*(TIMEUNITPERSECOND/STUFF->st_dacsr))));
     if (sampflag)
         x->c_unit = -timeunit;  /* negate to flag sample-based */
     else x->c_unit = timeunit * TIMEUNITPERMSEC;
@@ -157,7 +149,7 @@ double clock_gettimesincewithunits(double prevsystime,
             DSP ticks, the result will be exact. */
     if (sampflag)
         return ((pd_this->pd_systime - prevsystime)/
-            ((TIMEUNITPERSECOND/sys_dacsr)*units));
+            ((TIMEUNITPERSECOND/STUFF->st_dacsr)*units));
     else return ((pd_this->pd_systime - prevsystime)/(TIMEUNITPERMSEC*units));
 }
 
@@ -275,7 +267,7 @@ void glob_audiostatus(void)
 
         post("%9.2f\t%s",
             (sched_diddsp - oss_resync[nresyncphase].r_ntick)
-                * ((double)sys_schedblocksize) / sys_dacsr,
+                * ((double)STUFF->st_schedblocksize) / STUFF->st_dacsr,
             oss_errornames[errtype]);
         nresyncphase--;
     }
@@ -298,7 +290,7 @@ void sys_log_error(int type)
         sched_diored = 1;
     }
     sched_dioredtime =
-        sched_diddsp + (int)(sys_dacsr /(double)sys_schedblocksize);
+        sched_diddsp + (int)(STUFF->st_dacsr /(double)STUFF->st_schedblocksize);
 }
 
 static int sched_lastinclip, sched_lastoutclip,
@@ -314,12 +306,13 @@ static void sched_pollformeters( void)
         /* if there's no GUI but we're running in "realtime", here is
         where we arrange to ping the watchdog every 2 seconds. */
 #if defined(__linux__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__GNU__)
-    if (sys_nogui && sys_hipriority && (sched_diddsp - sched_nextpingtime > 0))
+    if (!sys_havegui() && sys_hipriority &&
+        (sched_diddsp - sched_nextpingtime > 0))
     {
         glob_watchdog(0);
             /* ping every 2 seconds */
         sched_nextpingtime = sched_diddsp +
-            2 * (int)(sys_dacsr /(double)sys_schedblocksize);
+            2 * (int)(STUFF->st_dacsr /(double)STUFF->st_schedblocksize);
     }
 #endif
 
@@ -354,7 +347,7 @@ static void sched_pollformeters( void)
         sched_lastoutdb = outdb;
     }
     sched_nextmeterpolltime =
-        sched_diddsp + (int)(sys_dacsr /(double)sys_schedblocksize);
+        sched_diddsp + (int)(STUFF->st_dacsr /(double)STUFF->st_schedblocksize);
 }
 
 void glob_meters(void *dummy, t_float f)
@@ -378,7 +371,6 @@ void dsp_tick(void);
 
 static int sched_useaudio = SCHED_AUDIO_NONE;
 static double sched_referencerealtime, sched_referencelogicaltime;
-double sys_time_per_dsp_tick;
 
 void sched_reopenmeplease(void)   /* request from s_audio for deferred reopen */
 {
@@ -401,15 +393,15 @@ void sched_set_using_audio(int flag)
                 post("sorry, can't turn off callbacks yet; restart Pd");
                     /* not right yet! */
 
-    sys_time_per_dsp_tick = (TIMEUNITPERSECOND) *
-        ((double)sys_schedblocksize) / sys_dacsr;
+    STUFF->st_time_per_dsp_tick = (TIMEUNITPERSECOND) *
+        ((double)STUFF->st_schedblocksize) / STUFF->st_dacsr;
     sys_vgui("pdtk_pd_audio %s\n", flag ? "on" : "off");
 }
 
     /* take the scheduler forward one DSP tick, also handling clock timeouts */
 void sched_tick( void)
 {
-    double next_sys_time = pd_this->pd_systime + sys_time_per_dsp_tick;
+    double next_sys_time = pd_this->pd_systime + STUFF->st_time_per_dsp_tick;
     int countdown = 5000;
     while (pd_this->pd_clock_setlist &&
         pd_this->pd_clock_setlist->c_settime < next_sys_time)
@@ -454,13 +446,10 @@ int (*sys_idlehook)(void);
 static void m_pollingscheduler( void)
 {
     int idlecount = 0;
-    sys_time_per_dsp_tick = (TIMEUNITPERSECOND) *
-        ((double)sys_schedblocksize) / sys_dacsr;
-
-#if THREAD_LOCKING
-        sys_lock();
-#endif
+    STUFF->st_time_per_dsp_tick = (TIMEUNITPERSECOND) *
+        ((double)STUFF->st_schedblocksize) / STUFF->st_dacsr;
 
+    sys_lock();
     sys_clearhist();
     if (sys_sleepgrain < 100)
         sys_sleepgrain = sys_schedadvance/4;
@@ -478,18 +467,14 @@ static void m_pollingscheduler( void)
     waitfortick:
         if (sched_useaudio != SCHED_AUDIO_NONE)
         {
-#if THREAD_LOCKING
-            /* T.Grill - send_dacs may sleep ->
-                unlock thread lock make that time available
-                - could messaging do any harm while sys_send_dacs is running?
-            */
             sys_unlock();
-#endif
             timeforward = sys_send_dacs();
-#if THREAD_LOCKING
-            /* T.Grill - done */
             sys_lock();
-#endif
+#if 0   /* in linux and windoes, sometimes audio devices would freeze, which
+               in turn would freeze Pd.  This code unfroze things by closing
+               audio in such cases.  But this seems no longer necessary, and
+               on Macs at least, this seems to cause audio to get dropped if
+               the machine sleeps.  */
                 /* if dacs remain "idle" for 1 sec, they're hung up. */
             if (timeforward != 0)
                 idlecount = 0;
@@ -517,6 +502,7 @@ static void m_pollingscheduler( void)
                     }
                 }
             }
+#endif /* 0 */
         }
         else
         {
@@ -546,9 +532,7 @@ static void m_pollingscheduler( void)
         {
             sched_pollformeters();
             sys_reportidle();
-#if THREAD_LOCKING
             sys_unlock();   /* unlock while we idle */
-#endif
                 /* call externally installed idle function if any. */
             if (!sys_idlehook || !sys_idlehook())
             {
@@ -556,17 +540,12 @@ static void m_pollingscheduler( void)
                 if (timeforward != SENDDACS_SLEPT)
                     sys_microsleep(sys_sleepgrain);
             }
-#if THREAD_LOCKING
             sys_lock();
-#endif
             sys_addhist(5);
             sched_didnothing++;
         }
     }
-
-#if THREAD_LOCKING
     sys_unlock();
-#endif
 }
 
 void sched_audio_callbackfn(void)
@@ -630,41 +609,13 @@ int m_mainloop(void)
 
 int m_batchmain(void)
 {
-    sys_time_per_dsp_tick = (TIMEUNITPERSECOND) *
-        ((double)sys_schedblocksize) / sys_dacsr;
+    STUFF->st_time_per_dsp_tick = (TIMEUNITPERSECOND) *
+        ((double)STUFF->st_schedblocksize) / STUFF->st_dacsr;
     while (sys_quit != SYS_QUIT_QUIT)
         sched_tick();
     return (0);
 }
 
-/* ------------ thread locking ------------------- */
-
-#if THREAD_LOCKING
-static pthread_mutex_t sys_mutex = PTHREAD_MUTEX_INITIALIZER;
-
-void sys_lock(void)
-{
-    pthread_mutex_lock(&sys_mutex);
-}
-
-void sys_unlock(void)
-{
-    pthread_mutex_unlock(&sys_mutex);
-}
-
-int sys_trylock(void)
-{
-    return pthread_mutex_trylock(&sys_mutex);
-}
-
-#else
-
-void sys_lock(void) {}
-void sys_unlock(void) {}
-int sys_trylock(void) {return (1);}
-
-#endif
-
 void sys_exit(void)
 {
     sys_quit = SYS_QUIT_QUIT;
