@@ -10,11 +10,20 @@
 # second while pd loads.
 if { [catch {wm withdraw .} fid] } { exit 2 }
 
-package require Tcl 8.3
-package require Tk
-#package require tile
-## replace Tk widgets with Ttk widgets on 8.5
-#namespace import -force ttk::*
+# This is mainly for OSX as older versions only
+# have 8.4 while newer versions have 8.5.
+if { [catch {package provide Tcl 8.5}] } {
+    # Tcl 8.5 not available
+    package require Tcl 8.4
+    package require Tk
+} else {
+    # Tcl 8.5 is available
+    package require Tcl 8.5
+    package require Tk
+
+    # replace Tk widgets with Ttk widgets on 8.5
+    namespace import -force ttk::*
+}
 
 package require msgcat
 # TODO create a constructor in each package to create things at startup, that
@@ -44,9 +53,9 @@ package require opt_parser
 package require pdtk_canvas
 package require pdtk_text
 package require pdtk_textwindow
+package require pd_guiprefs
 # TODO eliminate this kludge:
 package require wheredoesthisgo
-package require pd_guiprefs
 
 #------------------------------------------------------------------------------#
 # import functions into the global namespace
@@ -55,8 +64,9 @@ package require pd_guiprefs
 namespace import ::pd_guiprefs::init
 namespace import ::pd_guiprefs::update_recentfiles
 namespace import ::pd_guiprefs::write_recentfiles
-# make global since they are used throughout    
-namespace import ::pd_menucommands::* 
+
+# make global since they are used throughout
+namespace import ::pd_menucommands::*
 
 # import into the global namespace for backwards compatibility
 namespace import ::pd_connect::pdsend
@@ -75,7 +85,7 @@ namespace import ::dialog_array::pdtk_array_dialog
 namespace import ::dialog_audio::pdtk_audio_dialog
 namespace import ::dialog_canvas::pdtk_canvas_dialog
 namespace import ::dialog_data::pdtk_data_dialog
-namespace import ::dialog_find::pdtk_couldnotfind
+namespace import ::dialog_find::pdtk_showfindresult
 namespace import ::dialog_font::pdtk_canvas_dofont
 namespace import ::dialog_gatom::pdtk_gatom_dialog
 namespace import ::dialog_iemgui::pdtk_iemgui_dialog
@@ -123,20 +133,27 @@ set port 0
 set font_family "courier"
 set font_weight "normal"
 # sizes of chars for each of the Pd fixed font sizes:
-#  fontsize  width(pixels)  height(pixels)
-set font_fixed_metrics {
-    8 6 11
-    9 6 12
-    10 7 13
-    12 9 16
-    14 8 17
-    16 10 20
-    18 11 22
-    24 15 25
-    30 18 37
-    36 25 45
+# width(pixels)  height(pixels)
+set font_metrics {
+    6 10
+    7 13
+    9 16
+    10 21
+    15 25
+    25 45
 }
-set font_measured_metrics {}
+
+# sizes as above for zoomed-in view
+set font_zoom2_metrics {
+    12 20
+    14 26
+    18 32
+    20 42
+    30 50
+    50 90
+}
+set font_measured {}
+set font_zoom2_measured {}
 
 # root path to lib of Pd's files, see s_main.c for more info
 set sys_libdir {}
@@ -158,7 +175,6 @@ set startup_libraries {}
 set filenewdir [pwd]
 set fileopendir [pwd]
 
-
 # lists of audio/midi devices and APIs for prefs dialogs
 set audio_apilist {}
 set audio_indevlist {}
@@ -198,7 +214,7 @@ array set editingtext {};# if an obj, msg, or comment is being edited, per patch
 array set loaded {}     ;# store whether a patch has completed loading
 array set xscrollable {};# keep track of whether the scrollbars are present
 array set yscrollable {}
-# patch window tree, these might contain patch IDs without a mapped toplevel 
+# patch window tree, these might contain patch IDs without a mapped toplevel
 array set windowname {}    ;# window names based on mytoplevel IDs
 array set childwindows {}  ;# all child windows based on mytoplevel IDs
 array set parentwindows {} ;# topmost parent window ID based on mytoplevel IDs
@@ -217,19 +233,17 @@ set ::undo_action "no"
 set ::redo_action "no"
 set ::undo_toplevel "."
 
-
 namespace eval ::pdgui:: {
     variable scriptname [ file normalize [ info script ] ]
 }
 
-
 #------------------------------------------------------------------------------#
 # coding style
 #
 # these are preliminary ideas, we'll change them as we work things out:
 # - when possible use "" doublequotes to delimit messages
-# - use '$::myvar' instead of 'global myvar' 
-# - for the sake of clarity, there should not be any inline code, everything 
+# - use '$::myvar' instead of 'global myvar'
+# - for the sake of clarity, there should not be any inline code, everything
 #   should be in a proc that is ultimately triggered from main()
 # - if a menu_* proc opens a dialog panel, that proc is called menu_*_dialog
 # - use "eq/ne" for string comparison, NOT "==/!=" (http://wiki.tcl.tk/15323)
@@ -280,7 +294,7 @@ proc init_for_platform {} {
             option add *PatchWindow*Canvas.background "white" startupFile
             # add control to show/hide hidden files in the open panel (load
             # the tk_getOpenFile dialog once, otherwise it will not work)
-            catch {tk_getOpenFile -with-invalid-argument} 
+            catch {tk_getOpenFile -with-invalid-argument}
             set ::tk::dialog::file::showHiddenBtn 1
             set ::tk::dialog::file::showHiddenVar 0
             # set file types that open/save recognize
@@ -298,7 +312,7 @@ proc init_for_platform {} {
             # frame's upper left corner. http://wiki.tcl.tk/11502
             set ::windowframex 3
             set ::windowframey 53
-			# TODO add wm iconphoto/iconbitmap here if it makes sense
+            # TODO add wm iconphoto/iconbitmap here if it makes sense
             # mouse cursors for all the different modes
             set ::cursor_runmode_nothing "left_ptr"
             set ::cursor_runmode_clickme "arrow"
@@ -310,11 +324,20 @@ proc init_for_platform {} {
             set ::cursor_editmode_resize "sb_h_double_arrow"
         }
         "aqua" {
+            # load tk::mac event callbacks here, this way launching pd
+            # from the commandline incorporates the special mac event handling
+            package require apple_events
             set ::modifier "Mod1"
+            if {$::tcl_version < 8.5} {
+                # old default font for Tk 8.4 on macOS
+                # since font detection requires 8.5+
+                set ::font_family "Monaco"
+            }
             option add *DialogWindow*background "#E8E8E8" startupFile
             option add *DialogWindow*Entry.highlightBackground "#E8E8E8" startupFile
             option add *DialogWindow*Button.highlightBackground "#E8E8E8" startupFile
             option add *DialogWindow*Entry.background "white" startupFile
+            option add *DialogWindow*Menu.foreground "black" startupFile
             # Mac OS X needs a menubar all the time
             set ::dialog_menubar ".menubar"
             # set file types that open/save recognize
@@ -352,6 +375,9 @@ proc init_for_platform {} {
             option add *DialogWindow*font menufont startupFile
             option add *PdWindow*font menufont startupFile
             option add *ErrorDialog*font menufont startupFile
+            # initial dir is home
+            set ::filenewdir $::env(HOME)
+            set ::fileopendir $::env(HOME)
             # set file types that open/save recognize
             set ::filetypes \
                 [list \
@@ -364,7 +390,7 @@ proc init_for_platform {} {
             set ::menubarsize 0
             # Tk handles the window placement differently on each platform, on
             # Mac OS X, the x,y placement refers to the content window's upper
-            # left corner. http://wiki.tcl.tk/11502 
+            # left corner. http://wiki.tcl.tk/11502
             # TODO this probably needs a script layer: http://wiki.tcl.tk/11291
             set ::windowframex 0
             set ::windowframey 0
@@ -404,7 +430,7 @@ proc load_locale {} {
         }
     } elseif {$::tcl_platform(platform) eq "windows"} {
         # using LANG on Windows is useful for easy debugging
-        if {[info exists ::env(LANG)] && $::env(LANG) ne "C" && $::env(LANG) ne ""} {  
+        if {[info exists ::env(LANG)] && $::env(LANG) ne "C" && $::env(LANG) ne ""} {
             ::msgcat::mclocale $::env(LANG)
         } elseif {![catch {package require registry}]} {
             ::msgcat::mclocale [string tolower \
@@ -425,8 +451,8 @@ proc load_locale {} {
 # font handling
 
 # this proc gets the internal font name associated with each size
-proc get_font_for_size {size} {
-    return "::pd_font_${size}"
+proc get_font_for_size {fsize} {
+    return [list $::font_family -$fsize $::font_weight]
 }
 
 # searches for a font to use as the default.  Tk automatically assigns a
@@ -434,7 +460,7 @@ proc get_font_for_size {size} {
 # always do a good job of choosing in respect to Pd's needs.  So this chooses
 # from a list of fonts that are known to work well with Pd.
 proc find_default_font {} {
-    set testfonts {"DejaVu Sans Mono" "Bitstream Vera Sans Mono" \
+    set testfonts {"DejaVu Sans Mono" "Bitstream Vera Sans Mono" "Monaco" \
         "Inconsolata" "Courier 10 Pitch" "Andale Mono" "Droid Sans Mono"}
     foreach family $testfonts {
         if {[lsearch -exact -nocase [font families] $family] > -1} {
@@ -442,7 +468,7 @@ proc find_default_font {} {
             break
         }
     }
-    ::pdwindow::verbose 0 "Default font: $::font_family\n"
+    ::pdwindow::verbose 0 "Detected font: $::font_family\n"
 }
 
 proc set_base_font {family weight} {
@@ -461,41 +487,41 @@ proc set_base_font {family weight} {
             [_ "WARNING: Font weight '%s' not found, using default (%s)\n"] \
                 $weight $::font_weight]
     }
+    ::pdwindow::verbose 0 "Base font: $::font_family $::font_weight\n"
 }
 
-# creates all the base fonts (i.e. pd_font_8 thru pd_font_36) so that they fit
+# create all the base fonts (i.e. pd_font_8 thru pd_font_36) so that they fit
 # into the metrics given by $::font_fixed_metrics for any given font/weight
 proc fit_font_into_metrics {} {
-# TODO the fonts picked seem too small, probably on fixed width
-    foreach {size width height} $::font_fixed_metrics {
-        set myfont [get_font_for_size $size]
-        font create $myfont -family $::font_family -weight $::font_weight \
-            -size [expr {-$height}]
-        set height2 $height
-        set giveup 0
-        while {[font measure $myfont M] > $width || \
-            [font metrics $myfont -linespace] > $height} {
-            incr height2 -1
-            font configure $myfont -size [expr {-$height2}]
-            if {$height2 * 2 <= $height} {
-                set giveup 1
-                set ::font_measured_metrics $::font_fixed_metrics
-                break
-            }
+    set lastsize 0
+    set lastwidth 0
+    set lastheight 0
+
+    for {set fsize 6} {$fsize < 120 && [llength $::font_zoom2_metrics] > 1} \
+            {incr fsize} {
+        set foo [list $::font_family -$fsize bold]
+        set height [font metrics $foo -linespace]
+        set width [font measure $foo M]
+        # puts stderr [concat $fsize $width $height]
+        if {[llength $::font_metrics] > 1 && \
+            ( $width > [lindex $::font_metrics 0] || \
+            $height > [lindex $::font_metrics 1] )} {
+                # puts [concat SINGLE $fsize]
+                lappend ::font_measured $lastsize $lastwidth  $lastheight
+                set ::font_metrics [lrange $::font_metrics 2 end]
         }
-        set ::font_measured_metrics \
-            "$::font_measured_metrics  $size\
-                [font measure $myfont M] [font metrics $myfont -linespace]"
-        if {$giveup} {
-            ::pdwindow::post [format \
-    [_ "WARNING: %s failed to find font size (%s) that fits into %sx%s!\n"]\
-               [lindex [info level 0] 0] $size $width $height]
-            continue
+        if {$width > [lindex $::font_zoom2_metrics 0] || \
+            $height > [lindex $::font_zoom2_metrics 1]} {
+                # puts [concat DOUBLE $fsize]
+                lappend ::font_zoom2_measured $lastsize $lastwidth  $lastheight
+                set ::font_zoom2_metrics [lrange $::font_zoom2_metrics 2 end]
         }
+        set lastsize $fsize
+        set lastwidth $width
+        set lastheight $height
     }
 }
 
-
 # ------------------------------------------------------------------------------
 # procs called directly by pd
 
@@ -508,17 +534,21 @@ proc pdtk_pd_startup {major minor bugfix test
     set oldtclversion 0
     set ::audio_apilist $audio_apis
     set ::midi_apilist $midi_apis
+    ::pdwindow::verbose 0 "Tk [info patchlevel]\n"
     if {$::tcl_version >= 8.5} {find_default_font}
     set_base_font $sys_font $sys_fontweight
     fit_font_into_metrics
     ::pd_guiprefs::init
-    pdsend "pd init [enquote_path [pwd]] $oldtclversion $::font_measured_metrics"
+    pdsend "pd init [enquote_path [pwd]] $oldtclversion \
+        $::font_measured $::font_zoom2_measured"
     ::pd_bindings::class_bindings
     ::pd_bindings::global_bindings
     ::pd_menus::create_menubar
-    ::pdtk_canvas::create_popup
     ::pdwindow::create_window
+    ::pdwindow::configure_menubar
     ::pd_menus::configure_for_pdwindow
+    ::pdwindow::create_window_finalize
+    ::pdtk_canvas::create_popup
     load_startup_plugins
     open_filestoopen
     set ::done_init 1
@@ -540,9 +570,19 @@ proc pdtk_check {mytoplevel message reply_to_pd default} {
     }
 }
 
+# dispatch a message from running Pd patches to the intended plugin receiver
+proc pdtk_plugin_dispatch { args } {
+    set receiver [ lindex $args 0 ]
+    if [ info exists ::pd_connect::plugin_dispatch_receivers($receiver) ] {
+       foreach callback $::pd_connect::plugin_dispatch_receivers($receiver) {
+               $callback [ lrange $args 1 end ]
+       }
+    }
+}
+
 # ------------------------------------------------------------------------------
 # parse command line args when Wish/pd-gui.tcl is started first
- 
+
 proc parse_args {argc argv} {
     opt_parser::init {
         {-stderr    set {::stderr}}
@@ -554,7 +594,7 @@ proc parse_args {argc argv} {
         if { [string is int $argv] && $argv > 0} {
             # 'pd-gui' got the port number from 'pd'
             set ::host "localhost"
-            set ::port $argv 
+            set ::port $argv
         } else {
             set hostport [split $argv ":"]
             set ::port [lindex $hostport 1]
@@ -694,6 +734,11 @@ proc load_plugin_script {filename} {
 }
 
 proc load_startup_plugins {} {
+    # load built-in plugins
+    load_plugin_script [file join $::sys_guidir pd_deken.tcl]
+    load_plugin_script [file join $::sys_guidir pd_docsdir.tcl]
+
+    # load other installed plugins
     foreach pathdir [concat $::sys_searchpath $::sys_staticpath] {
         set dir [file normalize $pathdir]
         if { ! [file isdirectory $dir]} {continue}
@@ -708,8 +753,12 @@ proc load_startup_plugins {} {
 # ------------------------------------------------------------------------------
 # main
 proc main {argc argv} {
-    # TODO Tcl/Tk 8.3 doesn't have [tk windowingsystem]
     set ::windowingsystem [tk windowingsystem]
+    set ::platform $::tcl_platform(os)
+    if { $::tcl_platform(platform) eq "windows"} {
+       set ::platform W32
+    }
+
     tk appname pd-gui
     load_locale
     parse_args $argc $argv
@@ -726,11 +775,9 @@ proc main {argc argv} {
         set ::port [::pd_connect::create_socket]
         set pd_exec [file join [file dirname [info script]] ../bin/pd]
         exec -- $pd_exec -guiport $::port &
-        if {$::windowingsystem eq "aqua"} {
-            # on Aqua, if 'pd-gui' first, then initial dir is home
-            set ::filenewdir $::env(HOME)
-            set ::fileopendir $::env(HOME)
-        }
+        # if 'pd-gui' first, then initial dir is home
+        set ::filenewdir $::env(HOME)
+        set ::fileopendir $::env(HOME)
     }
     ::pdwindow::verbose 0 "------------------ done with main ----------------------\n"
 }
